# Parameter 'file_selectors' expects a named character vector of single 
# file dimension selectors.
# Parameter 'inner_indices' expects a named list of numeric or 
# character string vectors.
NcDimReader <- function(file_path = NULL, file_object = NULL, 
                        file_selectors = NULL, inner_indices = NULL) {
  if (!is.null(file_object)) {
    file_to_read <- file_object
  } else if (!is.null(file_path)) {
    file_to_read <- NcOpener(file_path)
  } else {
    stop("Either 'file_path' or 'file_object' must be provided.")
  }

  if (any(c('var', 'variable') %in% names(inner_indices))) {
    var_to_read <- inner_indices[[which(names(inner_indices) %in% c('var', 'variable'))[1]]]
    var_tag <- names(inner_indices)[[which(names(inner_indices) %in% c('var', 'variable'))[1]]]
  } else if (any(c('var', 'variable') %in% names(file_selectors))) {
    var_to_read <- file_selectors[[which(names(file_selectors) %in% c('var', 'variable'))[1]]]
    var_tag <- names(file_selectors)[[which(names(file_selectors) %in% c('var', 'variable'))[1]]]
  } else {
    stop("NcDimReader expected to find a requested 'var' or 'variable' in 'file_selectors'.")
  }

  vars_in_file <- easyNCDF::NcReadVarNames(file_to_read)
  if (var_to_read == 'var_names') {
    setNames(length(vars_in_file), var_tag)
  } else {
    var_to_read <- SelectorChecker(var_to_read, vars_in_file, return_indices = FALSE)
    read_dims <- easyNCDF::NcReadDims(file_to_read, var_to_read)
    if (any(c('var', 'variable') %in% names(inner_indices))) {
      names(read_dims)[which(names(read_dims) == 'var')] <- var_tag
      read_dims[var_tag] <- length(vars_in_file)
    } else {
      read_dims <- read_dims[-which(names(read_dims) == 'var')]
    }
    read_dims
  }
}
